MODULE MACHINE
  USE GDATA
  USE RAPPTURE, ONLY : Progress
  IMPLICIT NONE

  Character(Len=9) :: THEO ! Theoretical method used to calculate the WFN
  Integer          :: SG = 0 ! Abbort signal flag.
  Real(Kind=DP)    :: iT, fT ! Timing

CONTAINS

!> Print the error message "MSG" and orderly terminate the program.
  SUBROUTINE SUBEXIT(MSG)
    USE RAPPTURE, ONLY : Endl, Driver
    IMPLICIT NONE
! -- Dummy argument:
    Character(Len=*), Intent(IN) :: MSG 
! -- Local Variable:
    character(Len=132) :: STR
! ---
    SG = 1
    STR = "Job aborter by user."
    call rp_lib_put_str( driver,             &
                     "output.log",           &
                      endl//Trim(MSG)//endl, &
                     1 )
    CALL RP_RESULT(driver) 
    CALL EXIT(0)
  END SUBROUTINE SUBEXIT

SUBROUTINE RDPSI
  USE RAPPTURE, ONLY: Driver,str,endl
  IMPLICIT NONE
!
!  RDPSI READS AIMPAC WAVEFUNCTION FILE.  
!
!   INPUT CONSISTS OF;
!     MODE   - WAVEFUNCTION TYPE (SLATER OR GAUSSIAN)
!     NMO    - NO. OF MOLECULAR ORBITALS
!     NPRIMS - NO. OF (PRIMITIVE) BASIS FUNCTIONS
!     NCENT  - NO. OF NUCLEI
!
!  THEN FOR EACH NUCLEUS;
!     XC/YC/ZC   - COORDINATES
!     CHARG      - ATOMIC NUMBER
!
!  AND FOR EACH BASIS FUNCTION;
!     ICENT  - THE NO. OF THE NUCLEUS ON WHICH IT IS CENTERED
!     ITYPE  - FUNCTION TYPE (SEE ARRAYS LABELS AND LABELG)
!     EX     - EXPONENT
!
!  AND FOR EACH MOLECULAR ORBITAL;
!     PO    - OCCUPATION NUMBER
!     EORB  - ORBITAL ENERGY
!     CO    - COEFFICIENTS OF PRIMITIVE BASIS FUNCTIONS  
!             INCLUDING ALL NORMALIZATION AND CONTRACTION COEFFICIENTS
!
!  FOLOWED BY;
!     TOTE  - MOLECULAR ENERGY
!     GAMMA - VIRIAL RATIO (V/T)
!
  CHARACTER(Len=80)                    :: LINE
  CHARACTER(Len=4)                     :: MODE
  Integer      , DIMENSION(MPRIMS)     :: ITYPE, ICENT
  Real(Kind=DP), Dimension(MPRIMS)     :: EX
  Real(Kind=DP), Dimension(MPRIMS,MMO) :: CO

  Character(Len=81) :: REC  
  Integer :: P,GP=1
  Integer :: ioerr
  Integer :: I,J,K,L,N

!
!    THE ITYPE ARRAY REPRESENTS THE FOLLOWING GAUSSIAN ORBITAL TYPES:
!     S
!     PX, PY, PZ 
!     DXX, DYY, DZZ, DXY, DXZ, DYZ
!     FXXX, FYYY, FZZZ, FXXY, FXXZ, FYYZ, FXYY, FXZZ, FYZZ, FXYZ
!

!    READ IN WAVEFUNCTION TITLE
!
!  READ(IWFN,101) WFNTTL
  P   = INDEX(IWFN(GP:),achar(10))
  REC = IWFN(GP:GP+P-2)
  GP  = GP + P
  READ(REC,'(A)', IOSTAT=ioerr) WFNTTL
!ioerr = 1
  if ( ioerr /=0)  CALL SubExit("Error reading the TITLE in WFN file.")
!  call sleep(60)
!
!    READ IN MODE, NUMBER OF MOLECULAR ORBITALS, NUMBER OF PRIMITIVES,
!    AND NUMBER OF ATOMIC CENTERS
!
!--  READ(IWFN,102) MODE,NMO,NPRIMS,NCENT
!IWFN = IWFN(P+1:)
!P = INDEX(IWFN,achar(10))

  P   = INDEX(IWFN(GP:),achar(10))
 !REC = IWFN(GP:GP+P-1)  ! Includes the NL at the end of the record.
  REC = IWFN(GP:GP+P-2)  ! It does not include the NL. In practice, it does not matter.
  GP  = GP + P
  READ(REC,102,IOSTAT=ioerr) MODE,NMO,NPRIMS,NCENT
  if ( ioerr /=0)  CALL SubExit("Error detected reading 1st and 2nd line in WFN file.")

#ifdef DEBUG
  write(IDBG,'("Title found in WFN: ",A)'), WFNTTL
  write(IDBG,'(2A7,3(A8,I5))'), "MODE:  ",MODE, " NMO:   ", NMO, " NPRIMS: ",NPRIMS," NCENT: ",NCENT
#endif


  IF (NMO  > MMO) then
     STR = 'TOO MANY MOS. The parameter MMO must be increased.'//endl
     call rp_lib_put_str(driver, "output.log", Trim(STR), 1)
     call rp_result(driver) 
     CALL EXIT(0) ! see KILL and ABORT also.
!     Print*,'TOO MANY MOS. The parameter MMO must be increased.'
!     STOP
  endif
  IF (NPRIMS > MPRIMS) THEN
     STR = 'TOO MANY PRIMITIVES. Please increase the parameter MPRIMS.'//endl
     call rp_lib_put_str(driver, "output.log", Trim(STR), 1)
     call rp_result(driver) 
     CALL EXIT(0) 
!     STOP 'TOO MANY PRIMITIVES. Please increase the parameter MPRIMS.'
  END IF
  IF (NCENT  > MCENT)  THEN
     STR = 'TOO MANY NUCLEI. Please increase the parameter MCENT.'//endl
     call rp_lib_put_str(driver, "output.log", Trim(STR), 1)
     call rp_result(driver) 
     CALL EXIT(0)
!     STOP 'TOO MANY NUCLEI. Please increase the parameter MCENT.'
  END IF

!    READ IN ATOM NAMES, CARTESIAN COORDINATES, AND NUMBER OF
!    ELECTRONS PER ATOM
!
  DO I = 1, NCENT
     P   = INDEX(IWFN(GP:),achar(10))
     REC = IWFN(GP:GP+P-2)
     GP  = GP + P
     READ(REC,103) ATNAM(I),J,XC(J),YC(J),ZC(J),CHARG(J)
#ifdef DEBUG
     write(IDBG,'(A)') "Atomic coordinates successfully read:"
     write(IDBG,103) ATNAM(I),J,XC(J),YC(J),ZC(J),CHARG(J)
#endif
  END DO

!
!    READ IN FUNCTION CENTERS, FUNCTION TYPES, AND EXPONENTS
!

!  READ(IWFN,104) (ICENT(I), I=1,NPRIMS)
  DO i=1, CEILING(NPRIMS/20.)
     P   = INDEX(IWFN(GP:),achar(10))
     REC = IWFN(GP:GP+P-2)
     GP  = GP + P
     if (i <= CEILING(NPRIMS/20.)) then
        READ(REC,104) (ICENT(j), j=20*(i-1)+1, 20*i)
     else
        READ(REC,104) (ICENT(j), j=20*(i-1)+1, 20*(i-1) + MOD(NPRIMS,20))
     end if
  ENDDO

!  READ(IWFN,104) (ITYPE(I), I=1,NPRIMS)
  DO i=1, CEILING(NPRIMS/20.)
     P   = INDEX(IWFN(GP:),achar(10))
     REC = IWFN(GP:GP+P-2)
     GP  = GP + P
     if (i <= CEILING(NPRIMS/20.))then
        READ(REC,104) (ITYPE(j), j=20*i-19,20*i)
     else
        READ(REC,104) (ITYPE(j), j=20*i-19,20*(i-1) + MOD(NPRIMS,20))
     end if
  ENDDO

!  READ(IWFN,105) (EX(I),    I=1,NPRIMS)
  DO i=1,CEILING(NPRIMS/5.)
     P   = INDEX(IWFN(GP:),achar(10))
     REC = IWFN(GP:GP+P-2)
     GP  = GP + P
     if (i <= CEILING(NPRIMS/5.))then
        READ(REC,105) (EX(j), j=5*i-4,5*i)
     else
        READ(REC,105) (EX(j), j=5*i-4,5*(i-1) + MOD(NPRIMS,5))
     end if
  ENDDO
#ifdef DEBUG
  write(IDBG,'(A19)') "CENTER ASSIGNMENTS:"
  write(IDBG,104) (ICENT(I), I=1,NPRIMS)
  write(IDBG,'(A16)') "TYPE ASSIGNMENTS:"
  write(IDBG,104) (ITYPE(I), I=1,NPRIMS)
  write(IDBG,'(A10)') "EXPONENTS:"
  write(IDBG,105) (EX(I),    I=1,NPRIMS)
#endif

!
!    LOOP OVER MOLECULAR ORBITALS READING ORBITAL OCCUPANICES,
!    ORBITAL ENERGIES, AND COEFFICIENTS
!
  DO I = 1,NMO
   !     READ(IWFN,106) PO(I),EORB(I)
     P   = INDEX(IWFN(GP:),achar(10))
     REC = IWFN(GP:GP+P-1)
     GP  = GP + P
     READ(REC,106) PO(I), EORB(I)

   !  READ(IWFN,107) (CO(J,I),J=1,NPRIMS)
     DO J=1,CEILING(NPRIMS/5.0)
        P   = INDEX(IWFN(GP:),achar(10))
        REC = IWFN(GP:GP+P-1)
        GP  = GP + P
        if (j <= NPRIMS/5) then
           READ(REC,107) ( CO(k,i), k=5*j-4, 5*j )
        else
           READ(REC,107) ( CO(k,i), k=5*j-4, 5*(j-1) + MOD(NPRIMS,5))
        end if
     END DO
  END DO

#ifdef DEBUG
  DO I = 1,NMO
     write(IDBG,106) PO(I),EORB(I)
     write(IDBG,107) (CO(J,I),j=1,NPRIMS)
  end do
#endif

!
!  READ(IWFN,101) LINE
  P   = INDEX(IWFN(GP:),achar(10))
  REC = IWFN(GP:GP+P-1)
  GP  = GP + P
  READ(REC,101) LINE
#ifdef DEBUG
  write(IDBG,101) LINE
#endif
!
!    READ IN ENERGY AND -(V/T), THE VIRIAL RATIO
!
!READ(IWFN,109) TOTE,GAMMA
  P   = INDEX(IWFN(GP:),achar(10))
  REC = IWFN(GP:GP+P-1)
  GP  = GP + P
  READ(REC,109) THEO,TOTE,GAMMA
#ifdef DEBUG
  Print*, "Total Energy: ",TOTE,"  Virial Ratio: ",GAMMA
  write(IDBG,109) THEO,TOTE,GAMMA
#endif
!
  N=0
  DO K=1,NTYPE
     DO J=1,NPRIMS
        IF(ITYPE(J) == K) THEN
           N = N + 1
           EXX(N) = EX(J)
           ICT(N) = ICENT(J)
           SUM(N) = -2.0*EX(J)
           DIV(N) = 1./SUM(N)
           DO L=1,NMO
              COO(N,L)=CO(J,L)
           END DO
        ENDIF
     END DO
     ITP(K)=N
  END DO
!

!    FORMATS
101 FORMAT (A80)
102 FORMAT (4X,A4,11X,3(I4,16X))
103 FORMAT (A8,11X,I3,2X,3F12.8,10X,F5.1)
104 FORMAT (20X,20I3)
105 FORMAT (10X,5E14.7)
106 FORMAT (35X,F12.8,15X,F12.8)
107 FORMAT (5E16.8)
109 FORMAT (A9,8X,F20.12,18X,F13.8)
!109 FORMAT (17X,F20.12,18X,F13.8)
END SUBROUTINE RDPSI



SUBROUTINE CUBD2R(A,CX,XY)
  IMPLICIT NONE

!Dummy Variables
  Real(Kind=DP), DIMENSION(3,3), INTENT(IN) :: A
  Real(Kind=DP), DIMENSION(3),   INTENT(IN) :: CX
  Real(Kind=DP), DIMENSION(2),   INTENT(IN) :: XY


!Local Variables
Real(Kind=DP), DIMENSION(MPTS,3) :: XYZ
Real(Kind=DP), DIMENSION(MPTS)   :: SUMR

Integer           :: IZSTP, IYSTP, IXSTP
Real(KIND=DP)     :: XCN,YCN,ZCN, XCNT,YCNT,ZCNT
Integer           :: IX,IY, I,J

!
!    CALCULATE NUMBER OF STEPS IN X,Y AND Z FOR THIS CUBE AS WELL AS
!    INCREMENT MARKERS FOR CENTERING PLOT
!
IzSTP = INT(XY(1)/XY(2))
IYSTP = IzSTP
IxSTP = IzSTP
zCN   = -XY(1)/TWO
YCN   = zCN
xCN   = zCN
xCNT  = xCN
!

call WTIME(iT)
X:DO Ix = 1,IxSTP 
!
!    RESET Y INCREMENT
!
   YCNT = YCN 
!
   Y:DO IY = 1,IYSTP
!
!    RESET Z INCREMENT
!
      ZCNT = ZCN
!
      DO I = 1,IZSTP
         SUMR(I)=ZERO

!     BACK TRANSFORM PLANE INTO ORIGINAL MOLECULAR SYSTEM
          XYZ(I,1) = A(1,1)*XCNT + A(1,2)*YCNT + A(1,3)*ZCNT+CX(1)
          XYZ(I,2) = A(2,1)*XCNT + A(2,2)*YCNT + A(2,3)*ZCNT+CX(2)
          XYZ(I,3) = A(3,1)*XCNT + A(3,2)*YCNT + A(3,3)*ZCNT+CX(3)

          ZCNT = ZCNT + XY(2)
       END DO
!
       CALL GAUS(XYZ,IZSTP,0)
!
       DO J=1,NMO
          DO I = 1,IZSTP
             SUMR(I) = SUMR(I) + TWO*PO(J)*(PSI(I,J)*D2(I,J) + &
                  (GX(I,J)**2 + GY(I,J)**2 + GZ(I,J)**2))
          END DO
       END DO
!
!BTA       WRITE(ODX,FMT='(6E13.5)') (SNGL(SUMR(I)),I=1,IXSTP)
do i=1,iZstp
   if (ABS(SUMR(i)) > ISOCO) SUMR(i) = Sign(ISOCO,SUMR(i))
   WRITE(ODX,FMT='(E13.5)') REAL( SUMR(I) )
end do


!    INCREMENT Y VALUE
YCNT = YCNT + XY(2)

!BTA: Catch SIGTERM (id 15) in case of aborting job.
!           SIGABRT (is 6)  SIGKILL -> 9
   CALL SIGNAL(6,SubExit)
   if (SG /=0) EXIT X
!   CALL SIGNAL(15,KILL(PID=MyPID, SIGNAL=15, STATUS=stat))

END DO Y
!
!    INCREMENT X VALUE
XCNT = XCNT + XY(2)


!Inform user of the work progress:
!    CALL Progress(IX,IXSTP)
!BTA progress...
!Print 10, "Computation in progress....  Layer ",IX," out of ", INT(100*IX/IXSTP); call flush(5)
if (MOD(IX,10) == 0) then
   Print 20, "Computation in progress....  Layer ",IX," out of ",&
     IXSTP,".  ", INT(100*IX/IXSTP),"% done."
   call flush(5)
endif

!BTA: Timing 
   if(IX==1) THEN
      call WTIME(fT)
!      Print'(A,F8.2,A)',"BTA:",fT-iT," seconds."
      Print'(A,F8.2,A)',"Estimate computation time: ",IXSTP*(fT-iT)," seconds."
   endif

END DO X

10 FORMAT("[1G",A,I4,A,I4,$) ! This format does not work on the GUI as it is supposed to.
20 FORMAT(3(A,I4),A)
END SUBROUTINE CUBD2R


!
SUBROUTINE CUBKEG(A,CX,XY,IFUNC)
  IMPLICIT NONE

!Dummy Variables
Real(Kind=DP), DIMENSION(3,3), INTENT(IN) :: A
Real(Kind=DP), DIMENSION(3),   INTENT(IN) :: CX
Real(Kind=DP), DIMENSION(2),   INTENT(IN) :: XY
Integer, Intent(IN), Optional :: IFUNC

!Local Variables
Real(Kind=DP), DIMENSION(MPTS,3) :: XYZ
Real(Kind=DP), DIMENSION(MPTS)   :: SUMR
Real(KIND=DP)                    :: XCN,YCN,ZCN,  XCNT,ZCNT,YCNT
Integer                          :: IXSTP, IYSTP, IZSTP
Integer                          :: I,J, IX,IY

!
!    CALCULATE NUMBER OF STEPS IN X,Y AND Z FOR THIS CUBE AS WELL AS
!    INCREMENT MARKERS FOR CENTERING PLOT
!
IzSTP =  INT(XY(1)/XY(2))
IYSTP =  IzSTP
IxSTP =  IzSTP
zCN   = -XY(1)/TWO
YCN   =  zCN
xCN   =  zCN
xCNT  =  xCN
!

call WTIME(iT)
DO IX = 1,IXSTP 
!
!    RESET Y INCREMENT
!
   YCNT = YCN 
!
   DO IY = 1,IYSTP
!
!    RESET Z INCREMENT
!
      ZCNT = ZCN
!
      DO I = 1,IZSTP
!
         SUMR(I)=ZERO

!     BACK TRANSFORM PLANE INTO ORIGINAL MOLECULAR SYSTEM
!
         XYZ(I,1)=A(1,1)*XCNT+A(1,2)*YCNT+A(1,3)*ZCNT+CX(1)
         XYZ(I,2)=A(2,1)*XCNT+A(2,2)*YCNT+A(2,3)*ZCNT+CX(2)
         XYZ(I,3)=A(3,1)*XCNT+A(3,2)*YCNT+A(3,3)*ZCNT+CX(3)
!
         ZCNT=ZCNT+XY(2)
      END DO
!
      CALL GAUS(XYZ,IZSTP,0)
!
      DO J=1,NMO
         DO I = 1,IZSTP
            SUMR(I) = SUMR(I) + PT5*PO(J)*(GX(I,J)**2 + GY(I,J)**2 + GZ(I,J)**2)
         END DO
      END DO
!
      IF (IFUNC .EQ. 4) THEN
! CALCULA LA LAPLACIANA(RHO(R))
         DO I=1,IZSTP
            SUMR(I)=-TWO*SUMR(I)
         END DO

         DO J=1,NMO
            DO  I = 1,IZSTP
               SUMR(I)=SUMR(I)+(TWO*PO(J)*(PSI(I,J)*D2(I,J) + &
                    (GX(I,J)**2+GY(I,J)**2+GZ(I,J)**2)))/4.0_dp
            END DO
         END DO

      END IF
!BTA      WRITE(ODX,FMT='(6E13.5)') (SNGL(SUMR(I)),I=1,IZSTP)
!Print *," Debugging mode"; call flush(5)
do i=1,iZstp
if (ABS(SUMR(i)) > ISOCO) SUMR(i) = Sign(ISOCO,SUMR(i))
   WRITE(ODX,FMT='(E13.5)') REAL( SUMR(I) )
end do

!
!    INCREMENT Y VALUE
!
      YCNT=YCNT+XY(2)
!
   END DO
!
!    INCREMENT X VALUE
!
   XCNT = XCNT + XY(2)
!

! Inform user about the progress of the calculation:
   CALL Progress(IX,IXSTP)

!BTA: Catch SIGTERM (id 15) in case of aborting job.
   CALL Signal(6,SubExit)
!   if (SG /=0) EXIT

!BTA: Timing 
   if(IX==1) THEN
      call WTIME(fT)
!      Print'(A,F8.2,A)',"BTA:",fT-iT," seconds."
      Print'(A,F8.2,A)',"Estimate computation time: ",IXSTP*(fT-iT)," seconds."
   endif

 END DO

END SUBROUTINE CUBKEG

!
SUBROUTINE CUBRHO (A,CX,XY)
  USE RAPPTURE, ONLY : Progress, Sim_ID, Driver
  IMPLICIT NONE
!
!Dummy Variables
Real(Kind=DP), DIMENSION(3,3), INTENT(IN) :: A
Real(Kind=DP), DIMENSION(3),   INTENT(IN) :: CX
Real(Kind=DP), DIMENSION(2),   INTENT(IN) :: XY

!Local Variables
Real(Kind=DP), DIMENSION(MPTS,3) :: XYZ
Real(Kind=DP), DIMENSION(MPTS)   :: SUMR

Real(KIND=DP)     :: XCN,YCN,ZCN, XCNT,YCNT,ZCNT
Integer           :: IZSTP, IYSTP, IXSTP
Integer           :: IY,IX, I,J

! INTRINSIC KILL

! ---
!
!    CALCULATE NUMBER OF STEPS IN X,Y AND Z FOR THIS CUBE AS WELL AS
!    INCREMENT MARKERS FOR CENTERING PLOT
!
IZSTP =  INT(XY(1)/XY(2))
IYSTP =  IZSTP
IXSTP =  IZSTP
ZCN   = -XY(1)/TWO
YCN   =  ZCN
XCN   =  ZCN
XCNT  =  XCN
!

call WTIME(iT)
X: DO IX = 1,IXSTP 

   YCNT = YCN ! RESET Y INCREMENT

   Y: DO IY = 1,IYSTP

      ZCNT = ZCN ! RESET Z INCREMENT

      DO I = 1,IZSTP

         SUMR(I)=ZERO

!     BACK TRANSFORM PLANE INTO ORIGINAL MOLECULAR SYSTEM

         XYZ(I,1) = A(1,1)*xCNT + A(1,2)*YCNT + A(1,3)*zCNT + CX(1)
         XYZ(I,2) = A(2,1)*xCNT + A(2,2)*YCNT + A(2,3)*zCNT + CX(2)
         XYZ(I,3) = A(3,1)*xCNT + A(3,2)*YCNT + A(3,3)*zCNT + CX(3)

         ZCNT = ZCNT + XY(2)
      END DO

      CALL GAUS(XYZ,IZSTP,1)

      DO J = 1,NMO
         DO I = 1,IZSTP
            SUMR(I) = SUMR(I) + PO(J)*PSI(I,J)*PSI(I,J)
         END DO
      END DO
!
!BTA     WRITE(ODX,FMT='(6E13.5)')(SNGL(SUMR(I)),I=1,IXSTP) ! SNGL is archaic for REAL
!BTA      WRITE(ODX,FMT='(6E13.5)')(Real(SUMR(I)),I=1,IXSTP)
do i=1,iZstp
!BTA Rho>0   if (ABS(SUMR(i)) > ISOCO) SUMR(i) = Sign(ISOCO,SUMR(i))
if (SUMR(i)> ISOCO) SUMR(i)=ISOCO
   WRITE(ODX,FMT='(E13.5)') REAL( SUMR(I) )
end do
!
!    INCREMENT Y VALUE
!
        YCNT = YCNT + XY(2)

!BTA: Catch SIGTERM (id 15) in case of aborting job.
!           SIGABRT (is 6)  SIGKILL -> 9
   CALL SIGNAL(15,SubExit)
!   CALL SIGNAL(9,rp_result(driver)) ! OK but I need the interface to 'rp_result'
!   if (SG /=0) EXIT X
!   if (SG /=0) CALL EXIT(0)
   if (SG /=0) RETURN
!   CALL SIGNAL(15,KILL(Sim_id, 15))

     END DO Y
!    INCREMENT X VALUE
!
     XCNT = XCNT + XY(2)

!BTA: Inform user about the progress of the calculation:
   CALL Progress(IX,IXSTP)

!BTA: Timing 
   if(IX==1) THEN
      call WTIME(fT)
!      Print'(A,F8.2,A)',"BTA:",fT-iT," seconds."
      Print'(A,F8.2,A)',"Estimate computation time: ",IXSTP*(fT-iT)," seconds."
   endif
END DO X

END SUBROUTINE CUBRHO

! ----------------------------------------
SUBROUTINE EULER (E,A)
 IMPLICIT NONE
! 
!    CALCULATE THE ROTATION MATRICES FOR TRANSFORMATIONS 
!    BETWEEN THE X"Y" PLANE AND THE MOLECULAR COORDINATE 
!    SYSTEM.
!
!Dummy Variables
Real(Kind=DP), DIMENSION(3), Intent(IN)  ::  E
Real(Kind=DP), DIMENSION(9), Intent(OUT) ::  A

!Local Variables
Real(Kind=DP), Parameter :: Radian = 0.01745329251994329576923690768488612713443_dp ! 2PI/360
Real(Kind=DP) :: E1,E2,E3
Real(Kind=DP) :: SA,SB,SC, CA,CB,CC
!
!
!      PI=DACOS(-ONE)
!      RADIAN=TWO*PI/360
!
E1 = E(1)*RADIAN
E2 = E(2)*RADIAN
E3 = E(3)*RADIAN
!
SA = SIN(E1)
SB = SIN(E2)
SC = SIN(E3)
CA = COS(E1)
CB = COS(E2)
CC = COS(E3)
!
A(1) =  CC*CA - CB*SA*SC
A(2) =  CC*SA + CB*CA*SC
A(3) =  SC*SB
A(4) = -SC*CA - CB*SA*CC
A(5) = -SC*SA + CB*CA*CC
A(6) =  CC*SB
A(7) =  SB*SA
A(8) = -SB*CA
A(9) =  CB

END SUBROUTINE EULER

!

SUBROUTINE GAUS(XYZ,PTS,MM)
  IMPLICIT NONE

!Dummy Variables
Real(Kind=DP), Dimension(MPTS,3), Intent(IN) ::  XYZ
INTEGER, Intent(IN)                          ::  PTS,MM


!Local Variables:
Real(Kind=DP), Dimension(MPTS,MCENT)  :: Dx, Dy, Dz, R2
Real(Kind=DP), Dimension(MPTS,MPRIMS) :: Chi, ChiX, ChiY, ChiZ, ChiD2
Real(Kind=DP), Dimension(MPRIMS)      :: ChiMax

Real(Kind=DP), Parameter :: ONE = 1.0_dp, &
     THREE= 3.0_dp, FOUR= 4.0_dp, FIVE= 5.0_dp, &
     SIX = 6.0_dp, SEVEN=7.0_dp, NINE= 9.0_dp, CutOff=1.0D-10

REAL(KIND=DP) :: A, B, BXX, BXY, BXZ, BYZ, BYY, BZZ
REAL(KIND=DP) :: Check, Temp
Integer :: I,J,L, IS

!
SELECT CASE(MM)
CASE(0)

   DO J = 1,NCENT
      DO I = 1,PTS 
         DX(I,J) = XYZ(I,1) - XC(J)
         DY(I,J) = XYZ(I,2) - YC(J)
         DZ(I,J) = XYZ(I,3) - ZC(J)
         R2(I,J)= DX(I,J)*DX(I,J)+DY(I,J)*DY(I,J)+DZ(I,J)*DZ(I,J)
      END DO
   END DO
!
!       FOR S-TYPE
!
      DO J = 1,ITP(1) 
         IS=ICT(J)
         DO I=1,PTS 
            A = SUM(J)*EXP(-EXX(J)*R2(I,IS))
            CHI(I,J)   = A*DIV(J)
            CHIX(I,J)  = DX(I,IS)*A
            CHIY(I,J)  = DY(I,IS)*A
            CHIZ(I,J)  = DZ(I,IS)*A
            CHID2(I,J) = ( THREE + SUM(J)*R2(I,IS) )*A
         END DO
      END DO
!
!       FOR PX-TYPE
!
      DO J=ITP(1)+1,ITP(2)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DX(I,IS)*A*SUM(J)
!
            CHI(I,J)=A*DX(I,IS)
            CHIX(I,J)=A+DX(I,IS)*B
            CHIY(I,J)=DY(I,IS)*B
            CHIZ(I,J)=DZ(I,IS)*B
            CHID2(I,J)=(FIVE+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR PY-TYPE
!
      DO J=ITP(2)+1,ITP(3)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DY(I,IS)*A*SUM(J)
!
            CHI(I,J)=A*DY(I,IS)
            CHIX(I,J)=DX(I,IS)*B
            CHIY(I,J)=A+DY(I,IS)*B
            CHIZ(I,J)=DZ(I,IS)*B
            CHID2(I,J)=(FIVE+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR PZ-TYPE
!
      DO J=ITP(3)+1,ITP(4)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DZ(I,IS)*A*SUM(J)

            CHI(I,J)=A*DZ(I,IS)
            CHIX(I,J)=DX(I,IS)*B
            CHIY(I,J)=DY(I,IS)*B
            CHIZ(I,J)=A+DZ(I,IS)*B
            CHID2(I,J)=(FIVE+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DXX-TYPE
!
      DO J=ITP(4)+1,ITP(5)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DX(I,IS)*DX(I,IS)*A*SUM(J)
!
            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=(TWO*A+B)*DX(I,IS)
            CHIY(I,J)=DY(I,IS)*B
            CHIZ(I,J)=DZ(I,IS)*B
            CHID2(I,J)=TWO*A+(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DYY-TYPE
!
      DO J=ITP(5)+1,ITP(6)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DY(I,IS)*DY(I,IS)*A*SUM(J)
!
            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=DX(I,IS)*B
            CHIY(I,J)=(TWO*A+B)*DY(I,IS)
            CHIZ(I,J)=DZ(I,IS)*B
            CHID2(I,J)=TWO*A+(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DZZ-TYPE
!
      DO J=ITP(6)+1,ITP(7)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DZ(I,IS)*DZ(I,IS)*A*SUM(J)

            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=DX(I,IS)*B
            CHIY(I,J)=DY(I,IS)*B
            CHIZ(I,J)=(TWO*A+B)*DZ(I,IS)
            CHID2(I,J)=TWO*A+(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DXY-TYPE
!
      DO J=ITP(7)+1,ITP(8)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DX(I,IS)*DY(I,IS)*A*SUM(J)
!
            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=DX(I,IS)*B+DY(I,IS)*A
            CHIY(I,J)=DY(I,IS)*B+DX(I,IS)*A
            CHIZ(I,J)=DZ(I,IS)*B
            CHID2(I,J)=(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DXZ-TYPE
!
      DO J=ITP(8)+1,ITP(9)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DX(I,IS)*DZ(I,IS)*A*SUM(J)
!
            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=DX(I,IS)*B+DZ(I,IS)*A
            CHIY(I,J)=DY(I,IS)*B
            CHIZ(I,J)=DZ(I,IS)*B+DX(I,IS)*A
            CHID2(I,J)=(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR DYZ-TYPE
!
      DO J=ITP(9)+1,ITP(10)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DY(I,IS)*DZ(I,IS)*A*SUM(J)
!
            CHI(I,J)=B*DIV(J)
            CHIX(I,J)=DX(I,IS)*B
            CHIY(I,J)=DY(I,IS)*B+DZ(I,IS)*A
            CHIZ(I,J)=DZ(I,IS)*B+DY(I,IS)*A
            CHID2(I,J)=(SEVEN+SUM(J)*R2(I,IS))*B
         END DO
      END DO
!
!       FOR FXXX-TYPE
!
      DO J=ITP(10)+1,ITP(11)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DX(I,IS)*DX(I,IS)*A
!
            CHI(I,J)=B*DX(I,IS)
            CHIX(I,J)=(THREE + SUM(J)*DX(I,IS)*DX(I,IS))*B
            CHIY(I,J)=SUM(J)*DY(I,IS)*CHI(I,J)
            CHIZ(I,J)=SUM(J)*DZ(I,IS)*CHI(I,J)
            CHID2(I,J)=SIX*A*DX(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
         END DO
      END DO
!
!       FOR FYYY-TYPE
!
      DO J=ITP(11)+1,ITP(12)
         IS=ICT(J)
         DO I=1,PTS
            A=EXP(-EXX(J)*R2(I,IS))
            B=DY(I,IS)*DY(I,IS)*A
!
            CHI(I,J)=B*DY(I,IS)
            CHIX(I,J)=SUM(J)*DX(I,IS)*CHI(I,J)
            CHIY(I,J)=(THREE + SUM(J)*DY(I,IS)*DY(I,IS))*B
            CHIZ(I,J)=SUM(J)*DZ(I,IS)*CHI(I,J)
            CHID2(I,J)=SIX*A*DY(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
         END DO
      END DO
!
!       FOR FZZZ-TYPE
!
      DO J=ITP(12)+1,ITP(13)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            B=DZ(I,IS)*DZ(I,IS)*A
!
            CHI(I,J)=B*DZ(I,IS)
            CHIX(I,J)=SUM(J)*DX(I,IS)*CHI(I,J)
            CHIY(I,J)=SUM(J)*DY(I,IS)*CHI(I,J)
            CHIZ(I,J)=(THREE + SUM(J)*DZ(I,IS)*DZ(I,IS))*B
            CHID2(I,J)=SIX*A*DZ(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FXXY-TYPE
!
      DO J=ITP(13)+1,ITP(14)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BXY=DX(I,IS)*DY(I,IS)*A
            BXX=DX(I,IS)*DX(I,IS)*A
!
            CHI(I,J)=BXY*DX(I,IS)
            CHIX(I,J)=(TWO + SUM(J)*DX(I,IS)*DX(I,IS))*BXY
            CHIY(I,J)=(ONE + SUM(J)*DY(I,IS)*DY(I,IS))*BXX
            CHIZ(I,J)=SUM(J)*DZ(I,IS)*CHI(I,J)
            CHID2(I,J)=TWO*A*DY(I,IS)+  &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FXXZ-TYPE
!
      DO J=ITP(14)+1,ITP(15)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BXZ=DX(I,IS)*DZ(I,IS)*A
            BXX=DX(I,IS)*DX(I,IS)*A

            CHI(I,J)=BXZ*DX(I,IS)
            CHIX(I,J)=(TWO + SUM(J)*DX(I,IS)*DX(I,IS))*BXZ
            CHIY(I,J)=SUM(J)*DY(I,IS)*CHI(I,J)
            CHIZ(I,J)=(ONE + SUM(J)*DZ(I,IS)*DZ(I,IS))*BXX
            CHID2(I,J)=TWO*A*DZ(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FYYZ-TYPE
!
      DO J=ITP(15)+1,ITP(16)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BYZ=DZ(I,IS)*DY(I,IS)*A
            BYY=DY(I,IS)*DY(I,IS)*A
!
            CHI(I,J)=BYZ*DY(I,IS)
            CHIX(I,J)=SUM(J)*DX(I,IS)*CHI(I,J)
            CHIY(I,J)=(TWO + SUM(J)*DY(I,IS)*DY(I,IS))*BYZ
            CHIZ(I,J)=(ONE + SUM(J)*DZ(I,IS)*DZ(I,IS))*BYY
            CHID2(I,J)=TWO*A*DZ(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FXYY-TYPE
!
      DO J=ITP(16)+1,ITP(17)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BXY=DX(I,IS)*DY(I,IS)*A
            BYY=DY(I,IS)*DY(I,IS)*A

            CHI(I,J)=BXY*DY(I,IS)
            CHIX(I,J)=(ONE + SUM(J)*DX(I,IS)*DX(I,IS))*BYY
            CHIY(I,J)=(TWO + SUM(J)*DY(I,IS)*DY(I,IS))*BXY
            CHIZ(I,J)=SUM(J)*DZ(I,IS)*CHI(I,J)
            CHID2(I,J)=TWO*A*DX(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FXZZ-TYPE
!
      DO J=ITP(17)+1,ITP(18)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BXZ=DZ(I,IS)*DX(I,IS)*A
            BZZ=DZ(I,IS)*DZ(I,IS)*A
!
            CHI(I,J)=BXZ*DZ(I,IS)
            CHIX(I,J)=(ONE + SUM(J)*DX(I,IS)*DX(I,IS))*BZZ
            CHIY(I,J)=SUM(J)*DY(I,IS)*CHI(I,J)
            CHIZ(I,J)=(TWO + SUM(J)*DZ(I,IS)*DZ(I,IS))*BXZ
            CHID2(I,J)=TWO*A*DX(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FYZZ-TYPE
!
      DO J=ITP(18)+1,ITP(19)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BYZ=DZ(I,IS)*DY(I,IS)*A
            BZZ=DZ(I,IS)*DZ(I,IS)*A
!
            CHI(I,J)=BYZ*DZ(I,IS)
            CHIX(I,J)=SUM(J)*DX(I,IS)*CHI(I,J)
            CHIY(I,J)=(ONE + SUM(J)*DY(I,IS)*DY(I,IS))*BZZ
            CHIZ(I,J)=(TWO + SUM(J)*DZ(I,IS)*DZ(I,IS))*BYZ
            CHID2(I,J)=TWO*A*DY(I,IS)+ &
                 (NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
!       FOR FXYZ-TYPE
!
      DO J=ITP(19)+1,ITP(20)
         IS=ICT(J)
         DO I=1,PTS
!
            A=EXP(-EXX(J)*R2(I,IS))
            BXY=DX(I,IS)*DY(I,IS)*A
            BYZ=DZ(I,IS)*DY(I,IS)*A
            BXZ=DX(I,IS)*DZ(I,IS)*A
!
            CHI(I,J)=DX(I,IS)*BYZ
            CHIX(I,J)=(ONE + SUM(J)*DX(I,IS)*DX(I,IS))*BYZ
            CHIY(I,J)=(ONE + SUM(J)*DY(I,IS)*DY(I,IS))*BXZ
            CHIZ(I,J)=(ONE + SUM(J)*DZ(I,IS)*DZ(I,IS))*BXY
            CHID2(I,J)=(NINE+SUM(J)*R2(I,IS))*CHI(I,J)*SUM(J)
!
         END DO
      END DO
!
      DO J=1,NPRIMS
         TEMP=ZERO
         DO I=1,PTS
            CHECK = MAX( ABS(CHI(I,J)), ABS(CHIX(I,J)), ABS(CHIY(I,J)), &
                 ABS(CHIZ(I,J)), ABS(CHID2(I,J)),TEMP )
            IF(CHECK > TEMP) TEMP = CHECK
         END DO
         CHIMAX(J) = TEMP
      END DO


!
teje:      DO L= 1,NMO

         DO I= 1,PTS 
            PSI(I,L) = ZERO
            GX(I,L)  = ZERO
            GY(I,L)  = ZERO
            GZ(I,L)  = ZERO
            D2(I,L)  = ZERO
         END DO
!
         DO J = 1,NPRIMS
            CHECK=ABS(CHIMAX(J)*COO(J,L))
            IF(CHECK > CUTOFF)THEN
               TEMP=COO(J,L)
               DO I = 1,PTS
                  PSI(I,L) = PSI(I,L) + TEMP*CHI(I,J)
                  GX(I,L)  = GX(I,L)  + TEMP*CHIX(I,J)
                  GY(I,L)  = GY(I,L)  + TEMP*CHIY(I,J)
                  GZ(I,L)  = GZ(I,L)  + TEMP*CHIZ(I,J)
                  D2(I,L)  = D2(I,L)  + TEMP*CHID2(I,J)
               END DO
            ENDIF
         END DO

      END DO teje

!
CASE(1)
   DO J = 1,NCENT
      DO I=1,PTS 
         DX(I,J) = XYZ(I,1) - XC(J)
         DY(I,J) = XYZ(I,2) - YC(J)
         DZ(I,J) = XYZ(I,3) - ZC(J)
         R2(I,J)= DX(I,J)*DX(I,J)+DY(I,J)*DY(I,J)+DZ(I,J)*DZ(I,J)
      END DO
   END DO
!
!       FOR S-TYPE
!
       DO J = 1,ITP(1) 
          IS=ICT(J)
          DO I=1,PTS 
             CHI(I,J)=EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR PX-TYPE
!
       DO J=ITP(1)+1,ITP(2)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DX(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR PY-TYPE
!
       DO J=ITP(2)+1,ITP(3)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DY(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR PZ-TYPE
!
       DO J=ITP(3)+1,ITP(4)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DZ(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO

!
!       FOR DXX-TYPE
!
       DO J=ITP(4)+1,ITP(5)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DX(I,IS)*DX(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR DYY-TYPE
!
       DO J=ITP(5)+1,ITP(6)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DY(I,IS)*DY(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR DZZ-TYPE
!
       DO J=ITP(6)+1,ITP(7)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DZ(I,IS)*DZ(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR DXY-TYPE
!
       DO J=ITP(7)+1,ITP(8)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DX(I,IS)*DY(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR DXZ-TYPE
!
       DO J=ITP(8)+1,ITP(9)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DX(I,IS)*DZ(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR DYZ-TYPE
!
       DO J=ITP(9)+1,ITP(10)
          IS=ICT(J)
          DO I=1,PTS
             CHI(I,J)=DY(I,IS)*DZ(I,IS)*EXP(-EXX(J)*R2(I,IS))
          END DO
       END DO
!
!       FOR FXXX-TYPE
!
       DO J=ITP(10)+1,ITP(11)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DX(I,IS)**3
           END DO
        END DO
!
!       FOR FYYY-TYPE
!
        DO J=ITP(11)+1,ITP(12)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DY(I,IS)**3
           END DO
        END DO
!
!       FOR FZZZ-TYPE
!
        DO J=ITP(12)+1,ITP(13)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DZ(I,IS)**3
           END DO
        END DO
!
!       FOR FXXY-TYPE
!
        DO J=ITP(13)+1,ITP(14)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DY(I,IS)*DX(I,IS)**2
           END DO
        END DO
!
!       FOR FXXZ-TYPE
!
        DO J=ITP(14)+1,ITP(15)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DZ(I,IS)*DX(I,IS)**2
           END DO
        END DO
!
!       FOR FXYY-TYPE
!
        DO J=ITP(16)+1,ITP(17)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DX(I,IS)*DY(I,IS)**2
           END DO
        END DO
!
!       FOR FYYZ-TYPE
!
        DO J=ITP(15)+1,ITP(16)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DZ(I,IS)*DY(I,IS)**2
           END DO
        END DO
!
!       FOR FXZZ-TYPE
!
        DO J=ITP(17)+1,ITP(18)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DX(I,IS)*DZ(I,IS)**2
           END DO
        END DO
!
!       FOR FYZZ-TYPE
!
        DO J=ITP(18)+1,ITP(19)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DY(I,IS)*DZ(I,IS)**2
           END DO
        END DO
!
!       FOR FXYZ-TYPE
!
        DO J=ITP(19)+1,ITP(20)
           IS=ICT(J)
           DO I=1,PTS
              CHI(I,J)=EXP(-EXX(J)*R2(I,IS))*DX(I,IS)*DY(I,IS)*DZ(I,IS)
           END DO
        END DO
!
        DO J=1,NPRIMS
           TEMP=ZERO
           DO I=1,PTS
              CHECK=MAX(ABS(CHI(I,J)),TEMP)
              IF(CHECK > TEMP)TEMP=CHECK
           END DO
           CHIMAX(J)=TEMP
        END DO
!
        teje2: DO L = 1,NMO

           DO I=1,PTS 
              PSI(I,L) = ZERO
           END DO
!
           DO J = 1,NPRIMS
              CHECK=ABS(CHIMAX(J)*COO(J,L))
              IF(CHECK.GT.CUTOFF)THEN
                 TEMP=COO(J,L)
                 DO I = 1,PTS
                    PSI(I,L) = PSI(I,L) + TEMP*CHI(I,J)
                 END DO
              ENDIF
           END DO
        END DO teje2

 END SELECT
END SUBROUTINE GAUS
!
SUBROUTINE GROCKLE (N, X, IR, S, E)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      INTEGER, Intent(IN):: N
      DIMENSION X(3,N), S(3,N), C(3), E(3,3), EV(3), R(3,3)

      Integer :: I, M, IFAIL
      INTEGER IR(N)
      SAVE ZERO,ONE
      DATA ZERO /0.0D0/, ONE/1.0D0/
!
!    ZERO OUT CENTROID AND EIGENVECTORS
!
      C(1) = ZERO
      C(2) = ZERO
      C(3) = ZERO
      E(1,1) = ZERO
      E(1,2) = ZERO
      E(1,3) = ZERO
      E(2,2) = ZERO
      E(2,3) = ZERO
      E(3,3) = ZERO
!
!    CENTROID OF FRAGMENT
!
      M = 0
      DO I = 1, N
      IF (IR(I) > 0) THEN
        C(1) = C(1) + X(1,IR(I))
        C(2) = C(2) + X(2,IR(I))
        C(3) = C(3) + X(3,IR(I))
	M = M + 1
      ELSE IF (IR(I) < 0) THEN
        C(1) = C(1) + S(1,-IR(I))
        C(2) = C(2) + S(2,-IR(I))
        C(3) = C(3) + S(3,-IR(I))
	M = M + 1
      END IF
END DO
!
      DD = ONE/DFLOAT(M)
      C(1) = DD*C(1)
      C(2) = DD*C(2)
      C(3) = DD*C(3)
!
!    CALCULATE INERTIAL MATRIX.
!
      DO I = 1, N
      IF (IR(I) .GT. 0) THEN
        X1 = X(1,IR(I)) - C(1)
        X2 = X(2,IR(I)) - C(2)
        X3 = X(3,IR(I)) - C(3)
      ELSE IF (IR(I) .LT. 0) THEN
        X1 = S(1,-IR(I)) - C(1)
        X2 = S(2,-IR(I)) - C(2)
        X3 = S(3,-IR(I)) - C(3)
      END IF
      E(1,1) = E(1,1) + X2**2 + X3**2
      E(2,2) = E(2,2) + X1**2 + X3**2
      E(3,3) = E(3,3) + X1**2 + X2**2
      E(1,2) = E(1,2) - X1*X2
      E(1,3) = E(1,3) - X1*X3
      E(2,3) = E(2,3) - X2*X3
      X1 = ZERO
      X2 = ZERO
      X3 = ZERO
   END DO
!
      E(2,1) = E(1,2)
      E(3,1) = E(1,3)
      E(3,2) = E(2,3)
!
!    GENERATES EIGENVALUES AND EIGENVECTORS OF THE INERTIAL MATRIX.
!

      CALL TRACE(E, EV, C, 3, IFAIL)
!
!    CHECK FOR RIGHT HAND CONVENTION FOR EIGEN-AXES
!
      DET = E(1,1)*(E(2,2)*E(3,3) - E(3,2)*E(2,3)) + &
            E(1,2)*(E(3,1)*E(2,3) - E(2,1)*E(3,3)) + &
            E(1,3)*(E(2,1)*E(3,2) - E(3,1)*E(2,2))
!
      IF (DET < ZERO) THEN
        E(1,2) = -E(1,2)
        E(2,2) = -E(2,2)
        E(3,2) = -E(3,2)
      END IF
!

    END SUBROUTINE GROCKLE
!

SUBROUTINE TQLGRM(N, D, E, Z, IERR)
  IMPLICIT  DOUBLE PRECISION (A-H, O-Z)
  Integer, Intent(IN)  :: N
  Integer, Intent(OUT) :: IERR
  DIMENSION D(N), E(N), Z(N,N)
  PARAMETER (AMACH = 16.0E-13)
  PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)

Integer :: I, J, K, L, M, MML, II, L1
!
  IERR    = 0
  IF (N == 1) RETURN
!
  DO I = 2,N
     E(I-1)  = E(I)
  END DO
!
  F       = ZERO
  B       = ZERO
  E(N)    = ZERO
!
  DO L = 1,N
     J       = 0
     H       = AMACH*(ABS(D(L)) + ABS(E(L)))
     IF (B < H) B = H
! 
    DO M = L,N
        IF (ABS(E(M)) <= B) GOTO 120
     END DO
!
120 IF (M == L) GOTO 220
!
130 IF (J == 30) THEN
        IERR    = L
        RETURN
     END IF
!
     J       = J + 1
     L1      = L + 1
     G       = D(L)
     P       = (D(L1) - G)/(2*E(L))
     IF (ABS(P*AMACH) > ONE) THEN
        R       = P
     ELSE
        R       = SQRT(P*P + 1)
     END IF
     D(L)    = E(L)/(P + DSIGN(R,P))
     H       = G - D(L)
!
     DO I = L1,N
        D(I)    = D(I) - H
     END DO
!
     F       = F + H
     P       = D(M)
     C       = ONE
     S       = ZERO
     MML     = M - L
!
     DO II = 1,MML
        I       = M - II
        G       = C*E(I)
        H       = C*P
        IF (ABS(P) >= ABS(E(I))) THEN
           C       = E(I)/P
           R       = SQRT(C*C + 1)
           E(I+1)  = S*P*R
           S       = C/R
           C       = ONE/R
        ELSE
           C       = P/E(I)
           R       = SQRT(C*C + 1)
           E(I+1)  = S*E(I)*R
           S       = 1.D0/R
           C       = C*S
        END IF
        P       = C*D(I) - S*G
        D(I+1)  = H + S*(C*G + S*D(I))
!
        DO K = 1,N
           H       = Z(K,I+1)
           Z(K,I+1)= S*Z(K,I) + C*H
           Z(K,I)  = C*Z(K,I) - S*H
        END DO
!
     END DO
!
     E(L)    = S*P
     D(L)    = C*P
     IF (ABS(E(L)) > B) GOTO 130
!
220  D(L)    = D(L) + F
  END DO
!
BT2:  DO II = 2,N
     I       = II - 1
     K       = I
     P       = D(I)
!
     DO J = II,N
        IF (D(J) >= P) CYCLE
        K       = J
        P       = D(J)
     END DO
!
     IF (K == I) CYCLE BT2
     D(K)    = D(I)
     D(I)    = P
!
     DO J = 1,N
        P       = Z(J,I)
        Z(J,I)  = Z(J,K)
        Z(J,K)  = P
     END DO
!
  END DO BT2
END SUBROUTINE TQLGRM

!
SUBROUTINE TRACE(H, E, W, N, IERR)
!
! TRACE CALLS TREDIG AND TLQGRM TO DIAGONALIZE A SYMMETRIC REAL MATRIX.
! THE MATRIX IS PASSED DOWN IN H AND IS REPLACED BY THE EIGENVECTORS.
! THE EIGENVALUES IN E ARE STORED SMALLEST FIRST.
! THE WORK STORE W SHOULD BE AT LEAST OF DIMENSION N.
! SKK ==================================================================
!
  IMPLICIT DOUBLE PRECISION (A-H, O-Z)
  Integer, Intent(IN)   :: N
  Integer, Intent(OUT)  :: IERR
  DIMENSION H(N,N), E(N), W(N)
!
  CALL TREDIG	(N, E, W, H)
  CALL TQLGRM	(N, E, W, H, IERR)
!
  RETURN
END SUBROUTINE TRACE

!
SUBROUTINE TREDIG(N, D, E, Z)
  IMPLICIT DOUBLE PRECISION (A-H, O-Z)
  Integer, Intent(IN)  :: N
  DIMENSION       D(N), E(N), Z(N,N)
  PARAMETER	(ZERO = 0.0D0, ONE = 1.0D0)
  Integer :: I, J, K, L, II, JP1
!
  IF (N .EQ. 1) GOTO 320
!
  DO II = 2,N
     I       = N + 2 - II
     L       = I - 1
     H       = ZERO
     SCALE   = ZERO
!
     IF (L .LT. 2) GOTO 130
!
     DO K = 1,L
        SCALE   = SCALE + ABS(Z(I,K))
     END DO
!
     IF (SCALE .NE. ZERO) GOTO 140
130  E(I)    = Z(I,L)
     GOTO 290
!
140  RSCALE	= ONE/SCALE
     DO K = 1,L
        Z(I,K)  = Z(I,K)*RSCALE
        H       = H + Z(I,K)*Z(I,K)
     END DO
     F       = Z(I,L)
     G       = -DSIGN(DSQRT(H),F)
     E(I)    = SCALE*G
     H       = H - F*G
     Z(I,L)  = F - G
     F       = ZERO
     RH	= ONE/H
     RHSCALE	= RH*RSCALE	
!
     DO J = 1,L
        Z(J,I)  = Z(I,J)*RHSCALE
        G       = ZERO
!
        DO K = 1,J
           G       = G + Z(J,K)*Z(I,K)
        END DO
!
        JP1     = J + 1
        IF (L .LT. JP1) GOTO 220
!
        DO K = JP1,L
           G       = G + Z(K,J)*Z(I,K)
        END DO
!
220     E(J)    = G*RH
        F       = F + E(J)*Z(I,J)
     END DO
!
     HH      = F/(H + H)
!
     DO J = 1,L
        F       = Z(I,J)
        G       = E(J) - HH*F
        E(J)    = G
        DO K = 1,J
           Z(J,K)  = Z(J,K) - F*E(K) - G*Z(I,K)
        END DO
     END DO

!
     DO K	= 1,L
        Z(I,K)  =  SCALE*Z(I,K)
     END DO
!
290  D(I)    = H
  END DO
!
320 D(1)    = ZERO
  E(1)    = ZERO
!
  BT:  DO I = 1,N
     L = I - 1
     IF (D(I) == ZERO) GOTO 380
!
     DO J = 1,L
        G = ZERO
!
        DO K  = 1,L
           G = G + Z(I,K)*Z(K,J)
        END DO
!
        DO K = 1,L
           Z(K,J) = Z(K,J) - G*Z(K,I)
        END DO
!
     END DO

!
380  D(I)    = Z(I,I)
     Z(I,I)  = ONE
!     IF(L<1) GOTO 500
     IF(L<1) CYCLE BT
!
     DO J = 1,L
        Z(J,I)  = ZERO
        Z(I,J)  = ZERO
     END DO
!
  END DO BT
END SUBROUTINE TREDIG

END MODULE MACHINE
